package com.ur.urcap.externalaxis.coordinatedmotionprogrammer.impl;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JComboBox;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;

import com.ur.urcap.api.contribution.ContributionProvider;
import com.ur.urcap.api.contribution.program.swing.SwingProgramNodeView;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;
import com.ur.urcap.api.domain.program.structure.TreeStructureException;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;
import com.ur.urcap.api.contribution.ViewAPIProvider;

public
class CoordinatedProcessProgramNodeView extends BaseProcessMoveNodeView
    implements SwingProgramNodeView<CoordinatedProcessProgramNodeContribution> {
 private
  final Style style;

 private
  KeyboardInputFactory keyboardInputFactory;

 private
  JComboBox axisGroupComboBox = new JComboBox();

 private
  JComboBox trackedFrameComboBox = new JComboBox();

 public
  CoordinatedProcessProgramNodeView(ViewAPIProvider apiProvider,
                                    final Style style) {
    super(style, apiProvider.getUserInterfaceAPI()
                     .getUserInteraction()
                     .getKeyboardInputFactory());

    this.style = style;
    keyboardInputFactory = apiProvider.getUserInterfaceAPI()
                               .getUserInteraction()
                               .getKeyboardInputFactory();
  }

  @Override public void buildUI(
      JPanel panel,
      final ContributionProvider<CoordinatedProcessProgramNodeContribution>
          provider) {
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    panel.add(createGroupInputControls(provider));

    panel.add(createVerticalSpacing(style.getExtraLargeVerticalSpacing()));

    panel.add(createFrameTrackingInputControls(provider));

    panel.add(createVerticalSpacing(style.getExtraLargeVerticalSpacing()));

    panel.add(createParameterInputControls(provider));

    panel.add(createVerticalSpacing(style.getExtraLargeVerticalSpacing()));

    panel.add(createWaypointAddControls(provider));
  }

  // Adds a GUI panel for providing axis group and frame tracking info.
 private
  JPanel createGroupInputControls(
      final ContributionProvider<CoordinatedProcessProgramNodeContribution>
          provider) {
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    JLabel title = new JLabel("Axis Group");
    title.setFont(title.getFont()
                      .deriveFont((float)style.getSmallHeaderFontSize())
                      .deriveFont(Font.BOLD));
    panel.add(title);

    panel.add(createVerticalSpacing());

    panel.add(createLabelComboBox(
        "Select axis group to move:", axisGroupComboBox,
        new PopupMenuListener() {
          @Override public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
            axisGroupComboBox.removeAllItems();

            String[] names = provider.get().getAxisGroupNames();
            for (String node : names) {
              axisGroupComboBox.addItem(node);
            }
          }

          @Override public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
          }

          @Override public void popupMenuCanceled(PopupMenuEvent e) {
          }
        },

        new ItemListener() {
          @Override public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
              provider.get().saveAxisGroup(e.getItem().toString());
            }
          }
        }));

    return panel;
  }

 private
  JPanel createFrameTrackingInputControls(
      final ContributionProvider<CoordinatedProcessProgramNodeContribution>
          provider) {
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    JLabel title = new JLabel("Frame/Axis Tracking");
    title.setFont(title.getFont()
                      .deriveFont((float)style.getSmallHeaderFontSize())
                      .deriveFont(Font.BOLD));
    panel.add(title);

    panel.add(createVerticalSpacing());

    panel.add(createLabelComboBox(
        "Select what to track:", trackedFrameComboBox,
        new PopupMenuListener() {
          @Override public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
            trackedFrameComboBox.removeAllItems();

            String[] worldModelNames = provider.get().getAxisAndFrameNames();
            for (String node : worldModelNames) {
              trackedFrameComboBox.addItem(node);
            }
          }

          @Override public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
          }

          @Override public void popupMenuCanceled(PopupMenuEvent e) {
          }
        },

        new ItemListener() {
          @Override public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
              provider.get().saveTrackedFrame(e.getItem().toString());
            }
          }
        }));

    return panel;
  }

  // Adds a GUI panel for adding MovePWithAxisGroup or MoveCWithAxisGroup
  // waypoints to this node's subtree.
 private
  JPanel createWaypointAddControls(
      final ContributionProvider<CoordinatedProcessProgramNodeContribution>
          provider) {
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    JLabel title = new JLabel("Add Waypoint");
    title.setFont(title.getFont()
                      .deriveFont((float)style.getSmallHeaderFontSize())
                      .deriveFont(Font.BOLD));
    panel.add(title);

    panel.add(createVerticalSpacing());

    Box addMovePBox = Box.createHorizontalBox();
    addMovePBox.setAlignmentX(Component.LEFT_ALIGNMENT);

    JButton movePButton = new JButton("Add");

    movePButton.addActionListener(new ActionListener() {
      @Override public void actionPerformed(ActionEvent e) {
        // Generate a popup and don't add the child node if this node isn't
        // fully defined yet.
        if (provider.get().isDefined()) {
          provider.get().addMovePWithAxisGroup();
        } else {
          JOptionPane pane = new JOptionPane();
          pane.showMessageDialog(
              new JFrame(),
              "This node needs to be fully defined before adding a child.",
              "Error", JOptionPane.ERROR_MESSAGE);
        }
      }
    });

    addMovePBox.add(movePButton);
    addMovePBox.add(new JLabel("  MoveP with Axis"));

    panel.add(addMovePBox);

    panel.add(createVerticalSpacing());

    Box addMoveCBox = Box.createHorizontalBox();
    addMoveCBox.setAlignmentX(Component.LEFT_ALIGNMENT);

    JButton moveCButton = new JButton("Add");

    moveCButton.addActionListener(new ActionListener() {
      @Override public void actionPerformed(ActionEvent e) {
        // Generate a popup and don't add the child node if this node isn't
        // fully defined yet.
        if (provider.get().isDefined()) {
          provider.get().addMoveCWithAxisGroup();
        } else {
          JOptionPane pane = new JOptionPane();
          pane.showMessageDialog(
              new JFrame(),
              "This node needs to be fully defined before adding a child.",
              "Error", JOptionPane.ERROR_MESSAGE);
        }
      }
    });

    addMoveCBox.add(moveCButton);
    addMoveCBox.add(new JLabel("  MoveC with Axis"));

    panel.add(addMoveCBox);

    return panel;
  }

  // Creates a text input field with a label.
 public
  Box createLabelComboBox(final String label,
                          final JComboBox comboBox,
                          PopupMenuListener menuListener,
                          ItemListener itemListener) {
    Box horizontalBox = Box.createHorizontalBox();
    horizontalBox.setAlignmentX(Component.LEFT_ALIGNMENT);

    JLabel jLabel = new JLabel(label);

    comboBox.setPreferredSize(style.getInputfieldSize());
    comboBox.setMaximumSize(comboBox.getPreferredSize());
    comboBox.addPopupMenuListener(menuListener);
    comboBox.addItemListener(itemListener);

    horizontalBox.add(jLabel);
    horizontalBox.add(createHorizontalSpacing());
    horizontalBox.add(comboBox);

    return horizontalBox;
  }

 public
  void setTrackedFrameComboBox(final String trackedFrame) {
    if (trackedFrameComboBox.getItemCount() == 0) {
      trackedFrameComboBox.addItem(trackedFrame);
    }
    trackedFrameComboBox.setSelectedItem(trackedFrame);
  }

 public
  void setAxisGroupComboBox(final String axisGroup) {
    if (axisGroupComboBox.getItemCount() == 0) {
      axisGroupComboBox.addItem(axisGroup);
    }
    axisGroupComboBox.setSelectedItem(axisGroup);
  }
}
