package com.ur.urcap.externalaxis.coordinatedmotionprogrammer.impl;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.ur.urcap.api.contribution.ContributionProvider;
import com.ur.urcap.api.contribution.ViewAPIProvider;
import com.ur.urcap.api.contribution.program.swing.SwingProgramNodeView;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;

public class MoveCWithAxisGroupNodeView extends BaseMoveWithAxisGroupNodeView
    implements SwingProgramNodeView<MoveCWithAxisGroupNodeContribution> {

  private final Style style;

  private KeyboardInputFactory keyboardInputFactory;

  private JTextField robotViaPointInputField = new JTextField();

  public MoveCWithAxisGroupNodeView(ViewAPIProvider apiProvider, final Style style) {
    super(style, apiProvider.getUserInterfaceAPI()
        .getUserInteraction()
        .getKeyboardInputFactory());

    this.style = style;
    keyboardInputFactory = apiProvider.getUserInterfaceAPI()
        .getUserInteraction()
        .getKeyboardInputFactory();
  }

  public void setRobotViaPointInputField(final String value) {
    robotViaPointInputField.setText(value);
  }

  // Creates a GUI for setting the movec via point, the target robot pose, the
  // target axis pose, and the rest of the movec parameters.
  @Override
  public void buildUI(
      JPanel panel,
      final ContributionProvider<MoveCWithAxisGroupNodeContribution> provider) {

    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

    JPanel viaPointPanel = new JPanel();
    viaPointPanel.setLayout(new BoxLayout(viaPointPanel, BoxLayout.Y_AXIS));
    viaPointPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

    TitledBorder programViaBorder = BorderFactory.createTitledBorder(
        BorderFactory.createEtchedBorder(), "Program MoveC Via");
    programViaBorder.setTitleFont(
        programViaBorder.getTitleFont()
            .deriveFont((float) style.getLargeHeaderFontSize())
            .deriveFont(Font.BOLD));
    programViaBorder.setTitleJustification(TitledBorder.CENTER);

    viaPointPanel.setBorder(programViaBorder);
    viaPointPanel.add(createRobotViaInputControls(provider));

    JPanel waypointPanel = new JPanel();
    waypointPanel.setLayout(new BoxLayout(waypointPanel, BoxLayout.Y_AXIS));
    waypointPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

    TitledBorder programTargetBorder = BorderFactory.createTitledBorder(
        BorderFactory.createEtchedBorder(), "Program MoveC Target");
    programTargetBorder.setTitleFont(
        programTargetBorder.getTitleFont()
            .deriveFont((float) style.getLargeHeaderFontSize())
            .deriveFont(Font.BOLD));

    programTargetBorder.setTitleJustification(TitledBorder.CENTER);

    waypointPanel.setBorder(programTargetBorder);

    waypointPanel.add(createAxisInputControls(provider));
    waypointPanel.add(createVerticalSpacing(style.getVerticalSpacing()));
    waypointPanel.add(createRobotInputControls(provider));

    panel.add(viaPointPanel);
    panel.add(createVerticalSpacing(style.getVerticalSpacing()));
    panel.add(waypointPanel);

    panel.add(createVerticalSpacing(style.getVerticalSpacing()));

    panel.add(createParameterInputControls(provider));
  }

  // Creates a panel for setting/teaching the robot via pose.
  JPanel createRobotViaInputControls(
      final ContributionProvider<MoveCWithAxisGroupNodeContribution> provider) {

    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    panel.setAlignmentX(Component.LEFT_ALIGNMENT);

    JLabel paramsTitle = new JLabel("Robot Via Point (in tracked frame)");
    paramsTitle.setFont(paramsTitle.getFont()
        .deriveFont((float) style.getSmallHeaderFontSize())
        .deriveFont(Font.BOLD));
    panel.add(paramsTitle);

    panel.add(createVerticalSpacing());
    panel.add(createLabelInputField(
        "Target:", robotViaPointInputField, new MouseAdapter() {
          @Override
          public void mousePressed(MouseEvent e) {
            KeyboardTextInput keyboard = getKeyboard(provider.get().getRobotViaPoint());
            keyboard.show(robotViaPointInputField,
                provider.get().getCallbackForRobotViaPoint());
          }
        }));

    panel.add(createVerticalSpacing());

    JButton teachViaPointButton = new JButton("Set to Current Position");

    teachViaPointButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        provider.get().teachRobotViaPoint();
      }
    });

    panel.add(teachViaPointButton);

    return panel;
  }
}
