# Move-To URCap Example
## Description
The Move-To URCap provides a working example for moving the first axis in a given axis group via controls presented in the toolbar.
## Build
* The URCap SDK must be installed and available in the build environment
1.
   ```bash
   cd move-to
   ```
1.
   ```bash
   ./build.sh
   ```
   Note: By default the build script will attempt to build the URCap then deploy to a local simulator (if available).
## Usage
* Note: The Move-To URCap does not manage the lifecycle of any world-model or axis-group entities.
1. On the robot, execute a program or script that creates an Axis Group and Axis
   * The program/script should finish without removing the Axis Group. Remember to configure the axis as well, the EtherCAT Axis Config URCap could also be used for this purpose.
   * Example script:
     ```ruby
     POSITIONER_AXIS_TYPE = 0 # 0:revolute, 1:prismatic
     POSITIONER_VELOCITY_LIMIT = d2r(360) # rad/s
     POSITIONER_ACCELERATION_LIMIT = d2r(3600) # rad/s^2
     POSITIONER_ENCODER_RESOLUTION = 3600 # counts/rev
     POSITIONER_FEED_CONSTANT = d2r(360) # rad/rev
     POSITIONER_GEAR_RATIO = 1.0
     POSITIONER_ZERO_OFFSET = 0

     reset_world_model()
     axis_group_add("positioner", p[0, 0, 0, 0, 0, 0], "world")
     axis_group_add_axis("positioner", "axis1", "", p[0, 0, 0, 0, 0, 0], 
     POSITIONER_AXIS_TYPE, POSITIONER_VELOCITY_LIMIT, POSITIONER_ACCELERATION_LIMIT)
     ```
1. After the program has finished executing, open the Axis Group panel of the toolbar

   ![open toolbar](./docs/images/open_toolbar.png)
1. Enter the indicated parameters specific to the desired Axis Group to jog
   ![toolbar parameters](./docs/images/frame_tracking.png)
1. Move specified axes of the given Axis Group by holding down the 'Move axis to New Position' button.
   * The background color will change from white to grey indicating that the motion is in progress. The `Last Sent Command` field will update to indicate the command that has been sent to the robot via the Primary Interface
   * If the 'Frame Tracking' checkbox is checked, the robot will perform frame tracking as the axis group is moving.

   ![move axis group](./docs/images/moving_axis_group.png)
1. Halt jogging by releasing the held-down button
   * The axis group motion will halt, and the background color will change back to white indicating that the motion has stopped. 
