package com.ur.urcap.externalaxis.moveto.URCommunicator;

import java.io.*;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;

public class ScriptCommand {

    private final String primary_prefix = "def ";

    private final String programName;
    private final String postfix = "end\n";

    private String commandContent = "";

    /**
     * Create a new ScriptCommand
     */
    public ScriptCommand() {
        this.programName = "myCustomScript():\n";
    }

    /**
     * Create a new ScriptCommand with a custom name
     * 
     * @param commandName the custom name (must be alphanumeric, start with letter)
     */
    public ScriptCommand(String commandName) {
        this.programName = commandName + "():\n";
    }

    /**
     * Append a URScript line to the ScriptCommand
     * 
     * @param command the URScript line to append
     */
    public void appendLine(String command) {
        commandContent += " " + command + "\n";
    }

    public void appendFromFile(String filename) {
        try {
            InputStream in = getClass().getResourceAsStream(filename);
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String line = reader.readLine();
            while (line != null) {
                appendLine(line);
                line = reader.readLine();
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public String toString() {
        String command = "";
        command += primary_prefix;
        command += this.programName;
        command += this.commandContent;
        command += this.postfix;
        return command;
    }

}
