package com.ur.urcap.externalaxis.rotaryaxiscalibration.installationNode.installationNodeView;

import com.ur.urcap.externalaxis.rotaryaxiscalibration.installationNode.installationNodeContribution.interfaces.SamplePoseButtonListener;

import javax.swing.*;
import java.awt.*;

public class SamplePosePanel extends JPanel {
    private final RotaryAxisCalibrationInstallationNodeView view;
    private SamplePoseButtonListener samplePoseButtonListener;
    private JTextPane instructionsTextPane;
    private JTextPane sampledPosesTextPane;
    private JButton samplePoseButton;
    private JProgressBar progressBar;
    private JLabel sampledPosesLabel;
    private JScrollPane sampledPosesScrollPane;
    private String existingText = "";

    private int clickCount = 0;
    static String poseArray = "";

    private static final int progressBarMax = 4;

    public SamplePosePanel(RotaryAxisCalibrationInstallationNodeView view) {
        this.view = view;
        initComponents();
    }

    private void initComponents() {

        instructionsTextPane = new JTextPane();
        sampledPosesTextPane = new JTextPane();
        sampledPosesScrollPane = new JScrollPane();
        samplePoseButton = new JButton();
        progressBar = new JProgressBar();
        sampledPosesLabel = new JLabel();

        instructionsTextPane.setContentType("text/html");
        instructionsTextPane.setText("<html><body>" +
                "<p><b>Calibration Steps:</b> </p>" +
                "Calibration involves sampling at least 4 points around the rotation axis. " +
                "The four points must lie on a circle - i.e. on a plane with equal distance " +
                "from the center of the axis of rotation. " +
                "The points should be approximately 90 degrees apart. If looking at a clock," +
                "the sample points would be at 3, 12, 9 and 6 o'clock, in that order. "+
                "<ol>" +
                "<li>Hold down the freedrive button on the Teach Pendant</li>" +
                "<li>Move the robot to the desired point and release the freedrive button</li>" +
                "<li>Press the Sample button below</li>" +
                "<li>Repeat this process until at least 4 points have been sampled.</li>" +
                "<li>Press the Calibrate Axis button.</li>" + "</ol>" +
                "</body></html>");
        instructionsTextPane.setFocusable(false);
        sampledPosesScrollPane.setViewportView(sampledPosesTextPane);
        sampledPosesScrollPane.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        sampledPosesTextPane.setFocusable(true);

        samplePoseButton.setText("Sample Pose");
        samplePoseButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                samplePoseButtonTab2MouseClicked(evt);
            }
        });

        progressBar.setMaximum(4);
        progressBar.setToolTipText("");
        UIManager.put("blue", new Color(38, 139, 210));
        progressBar.setString(0 + "/"+progressBarMax);
        progressBar.setRequestFocusEnabled(true);
        progressBar.setStringPainted(true);

        sampledPosesLabel.setText("Sampled poses:");

        JPanel jPanel1 = new JPanel();
        BoxLayout box = new BoxLayout(jPanel1, BoxLayout.Y_AXIS);

        int width = 480;
        int height = 626;
        sampledPosesScrollPane.setPreferredSize(new Dimension(width, 230));
        sampledPosesScrollPane.setMaximumSize(new Dimension(width, 230));
        sampledPosesScrollPane.setMinimumSize(new Dimension(width, 230));

        jPanel1.setPreferredSize(new Dimension(width, height));
        jPanel1.setMaximumSize(new Dimension(width, height));
        jPanel1.setMinimumSize(new Dimension(width, height));

        jPanel1.setLayout(box);
        instructionsTextPane.setMaximumSize(new Dimension(width, 255));
        jPanel1.add(instructionsTextPane);


        FlowLayout flowLayout = new FlowLayout(FlowLayout.LEFT);
        JPanel flowPanel = new JPanel(flowLayout);
        flowPanel.setMaximumSize(new Dimension(width, sampledPosesLabel.getHeight()));
        flowPanel.add(sampledPosesLabel);

        jPanel1.add(flowPanel);

        jPanel1.add(sampledPosesScrollPane);
        jPanel1.add(Box.createRigidArea(new Dimension(0, 20)));

        JButton clearButton = new JButton("Clear");
        clearButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                poseArray = "";
                existingText = "";
                sampledPosesTextPane.setText("");
                progressBar.setValue(0);
                progressBar.setString(0 + "/" + progressBarMax);
                clickCount = 0;
                view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(false);
                view.getCalibrationResultPanel().getCalibrationResultLabel().setText("No data yet");
                view.getCalibrationResultPanel().getCalibrationCommandLabel().setText("No data yet");
            }
        });

        Box horizontalBox = Box.createHorizontalBox();
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(samplePoseButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(progressBar);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(clearButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));

        jPanel1.add(horizontalBox);

        add(jPanel1);
    }

    public void setButtonClickListener(SamplePoseButtonListener listener) {
        this.samplePoseButtonListener = listener;
    }

    private void samplePoseButtonTab2MouseClicked(java.awt.event.MouseEvent evt) {
        clickCount++;
        progressBar.setValue(clickCount);
        progressBar.setString(clickCount + "/" + progressBarMax);
        if (clickCount >= progressBarMax) {
            view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(true);
        }
        samplePoseButtonListener.onSamplePoseButtonClicked();
    }

    public void setLabelText(String text) {
        text = text.replaceAll("(\\d+\\.\\d{5})\\d+", "$1");

        sampledPosesTextPane.setContentType("text/html");
        if (!existingText.isEmpty()) {
            poseArray += ", ";
            poseArray += "p"+text;
            existingText = existingText.concat("<br>p" + text + "<br>");
        }
        else {
            poseArray += "p"+text;
            existingText = existingText.concat("p" + text + "<br>");
        }
        sampledPosesTextPane.setText(existingText);
    }

    public String getPoseArray() {
        return poseArray;
    }
}




